// Copyright 2012 MightyClient Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a mock implementation of mc.Channel.
 */


goog.require('goog.testing.recordFunction');
goog.require('mc.Channel');

goog.provide('mc.testing.Channel');



/**
 * A mock implementation of mc.Channel.
 * @param {boolean=} opt_initiallyClosed Whether the channel should be in closed
 *     state after it is instantiated. By default, the channel is open.
 * @constructor
 * @extends {mc.Channel}
 */
mc.testing.Channel = function(opt_initiallyClosed) {
  goog.base(this);


  /**
   * The value returned by isOpen().
   * @type {boolean}
   * @private
   */
  this.open_ = !opt_initiallyClosed;


  this.sendInternal = goog.testing.recordFunction();
};
goog.inherits(mc.testing.Channel, mc.Channel);


/** @inheritDoc */
mc.testing.Channel.prototype.isOpen = function() {
  return this.open_;
};


/**
 * Simulates opening of the channel.
 */
mc.testing.Channel.prototype.simulateOpen = function() {
  this.open_ = true;
  this.dispatchEvent(mc.Channel.EventType.OPEN);
};


/**
 * Simulates closing of the channel.
 */
mc.testing.Channel.prototype.simulateClose = function() {
  this.open_ = false;
  this.dispatchEvent(mc.Channel.EventType.CLOSE);
};
