// Copyright 2012 MightyClient Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a mock implementation of mc.GaeChannel.
 */


goog.require('mc.GaeChannel');
goog.require('mc.testing.Channel');

goog.provide('mc.testing.GaeChannel');



/**
 * A mock implementation of mc.GaeChannel.
 * @constructor
 * @extends {mc.testing.Channel}
 */
mc.testing.GaeChannel = function() {
  goog.base(this, true);


  /**
   * A stub for the real method. simulateOpen needs to be called to actually
   * open the channel.
   */
  this.open = goog.testing.recordFunction(function() {
    goog.asserts.assert(!this.isOpen(), 'Channel already open.');
  });
};
goog.inherits(mc.testing.GaeChannel, mc.testing.Channel);


/**
 * Simulates start of automatic reopening (should be followed by a call to
 * simulateOpenFailure or simulateOpen).
 */
mc.testing.GaeChannel.prototype.simulateAutoOpenStart = function() {
  this.dispatchEvent(mc.GaeChannel.EventType.AUTO_OPEN_START);
};


/**
 * Simulates OPEN_FAILURE.
 */
mc.testing.GaeChannel.prototype.simulateOpenFailure = function() {
  this.dispatchEvent(mc.GaeChannel.EventType.OPEN_FAILURE);
};
