#!/usr/bin/python
# Copyright 2012 MightyClient Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Compiles client/client.js into compiled_scripts/client.js using Closure
Compiler web service.
"""

import httplib
import json
import os
import pprint
import urllib

APP_DIR = os.path.normpath(os.path.join(__file__, '..', '..'))
SCRIPT_NAMES = [
  os.path.join('lib', 'client', 'mc', 'channel.js'),
  os.path.join('lib', 'client', 'mc', 'datasource.js'),
  os.path.join('lib', 'client', 'mc', 'gaechannel.js'),
  os.path.join('lib', 'client', 'mc', 'serveragent.js'),
  os.path.join('client', 'client.js')
]

scripts = []
for script_name in SCRIPT_NAMES:
  with open(os.path.join(APP_DIR, script_name)) as f:
    scripts.append(('js_code', f.read()))

with open(os.path.join(APP_DIR, 'lib', 'client', 'mc',
    'gaechannel_externs.js')) as f:
  externs = f.read()

params = urllib.urlencode(scripts + [
    ('compilation_level', 'ADVANCED_OPTIMIZATIONS'),
    ('js_externs', externs),
    ('use_closure_library', 'true'),
    ('output_format', 'json'),
    ('output_info', 'compiled_code'),
    ('output_info', 'warnings'),
    ('output_info', 'errors'),
    ('output_info', 'statistics'),
  ])

headers = { "Content-type": "application/x-www-form-urlencoded" }
conn = httplib.HTTPConnection('closure-compiler.appspot.com')
conn.request('POST', '/compile', params, headers)
response = conn.getresponse()
data = json.load(response)
conn.close()
if 'compiledCode' in data:
  if not os.path.exists(os.path.join(APP_DIR, 'compiled_scripts')):
    os.makedirs(os.path.join(APP_DIR, 'compiled_scripts'))
  filename = os.path.join(APP_DIR, 'compiled_scripts', 'client.js')
  with open(filename, 'w') as f:
    f.write(data['compiledCode'])
    print 'Saved compiled script as {}.'.format(filename)
  del data['compiledCode']
pprint.pprint(data)
