import os
import random
import string

import webapp2
import markupsafe
from webapp2_extras import jinja2

from lib.server.mc.channel import ForwardChannelHandler


class BaseHandler(webapp2.RequestHandler):
  @webapp2.cached_property
  def jinja2(self):
    return jinja2.get_jinja2(app=self.app)

  def render_template(self, filename, **template_args):
    self.response.write(self.jinja2.render_template(filename, **template_args))


def get_new_datasource_id():
  return ''.join(random.choice(string.ascii_letters +
      string.digits) for x in range(12))


class IndexHandler(BaseHandler):
  def get(self):
    if self.request.get('dev', default_value=None) is None:
      dev = ''
    else:
      dev = '&dev'
    datasource_id = self.request.get('datasource_id', default_value=None)
    if datasource_id is None:
      datasource_id = get_new_datasource_id()
      return self.redirect('?datasource_id=' + datasource_id + dev)
    self.render_template('index.html', datasource_id=datasource_id,
        dev=markupsafe.Markup(dev))


class HeaderHandler(BaseHandler):
  def get(self):
    self.render_template('header.html')


class ClientHandler(BaseHandler):
  def get(self):
    if self.request.get('dev', default_value=None) is None:
      dev = ''
      filename = os.path.join('compiled_scripts', 'client.js')
      try:
        with open(filename) as f:
            compiled = markupsafe.Markup(f.read())
      except IOError:
        raise Exception("Could not read {}, run {} to generate it.".format(
            filename, os.path.join('bin', 'compile.py')))
    else:
      dev = '&dev'
      if not os.environ.get('SERVER_SOFTWARE', '').startswith('Dev'):
        webapp2.abort(400, 'Uncompiled mode is only available on Dev server.')
      compiled = ''
    datasource_id = self.request.get('datasource_id', default_value=None)
    if datasource_id is None:
      datasource_id = get_new_datasource_id()
      return self.redirect('?datasource_id=' + datasource_id + dev)
    self.render_template('client.html', datasource_id=datasource_id,
        compiled=compiled)


app = webapp2.WSGIApplication([
  ('/', IndexHandler),
  ('/header', HeaderHandler),
  ('/_client', ClientHandler),
  ('/mc_channel', ForwardChannelHandler),
], debug=True)
