# Copyright 2012 MightyClient Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""Defines classes SerializedPayload and Service.
"""


class SerializedPayload(object):
  """A class for objects containing a payload already serialized to JSON.

  Supports == and != for the purposes of unit tests: two objects are equal iff
  they are both instances of SerializedPayload and wrap equal strings.

  Attributes:
      payload: serialized payload.
  """

  def __init__(self, payload):
    """
    Args:
        payload: a JSON-encoded string.
    """
    self.payload = payload


  def __eq__(self, other):
    return (isinstance(other, SerializedPayload) and
        self.payload == other.payload)


  def __ne__(self, other):
    return not self.__eq__(other)


  def __repr__(self):
    return 'SerializedPayload({})'.format(repr(self.payload))


class Service(object):
  """Base class for channel services.
  """

  def __init__(self, name, client_id, request):
    self.__name = name
    self.__client_id = client_id
    self.__request = request


  @property
  def name(self):
    """Name of the service for which the class was instantiated.
    """
    return self.__name


  @property
  def client_id(self):
    """ID of the client for which the service has been instantiated.
    """
    return self.__client_id


  @property
  def request(self):
    """The forward channel request, an instance of webapp2.Request.
    """
    return self.__request


  def handle_message(self, payload):
    """Handles an incoming message.

    Args:
        payload: deserialized payload.
    """
    pass


  def send(self, payload, service=None, to=None):
    """Sends a message to one or more clients.

    A client should have opened the channel in order to receive the message,
    unless the message is sent while a forward channel request from this client
    is being processed on the same server.

    Args:
        payload: payload that can be serialized to JSON or a SerializedPayload.
        service: name of the service, self.name by default.
        to: ID of the client to which the message should be sent, or a list of
            IDs to send the same message to multiple clients. self.client_id by
            default.
    """
    import channel
    if service is None: service = self.name
    if to is None: to = self.client_id
    channel.send(payload, service, to)
