#!/usr/bin/python
#
# Copyright 2012 MightyClient Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import string
import unittest

from google.appengine.api import memcache
from google.appengine.ext import testbed

import memcache_utils


class TestCase(unittest.TestCase):

  def setUp(self):
    self.testbed = testbed.Testbed()
    self.testbed.activate()
    self.time = 0.
    stub = memcache.memcache_stub.MemcacheServiceStub(
        gettime=(lambda: self.time))
    self.testbed._register_stub(testbed.MEMCACHE_SERVICE_NAME, stub)


  def tearDown(self):
    self.testbed.deactivate()


  def test_roundtrip(self):
    self.assertEquals('abc', memcache_utils.maybe_split('abc'))
    self.assertEquals('abc', memcache_utils.maybe_merge('abc'))
    long_string = ''.join((char*100000 for char in string.ascii_lowercase))
    self.assertEquals(2600000, len(long_string))
    value = memcache_utils.maybe_split(long_string, time=100)
    self.assertTrue(isinstance(value, list))
    self.assertEquals(3, len(value))
    self.time = 99.
    self.assertEquals(long_string, memcache_utils.maybe_merge(value))
    self.time = 101.
    with self.assertRaises(memcache_utils.MemcacheReadFailure):
      memcache_utils.maybe_merge(value)


if __name__ == '__main__':
  unittest.main()
