# Client-server communication protocol: channel #

## Messages sent from client to server ##

These messages are sent either through POSTs with the following properties:

- URL path: the value of `mc.GaeChannel.BASE_URL_PATH` on the client and `channel._config.BASE_URL_PATH` on the server.

- Query parameters:

    - `zx` (value of `goog.Uri.RANDOM_PARAM`): a random value to make URI unique.

- Post parameters:

    - `cid`: client ID.

    - `m`: messages, a JSON-encoded object where keys are service names and values are arrays of messages.

- Headers:

    - `Content-Type: application/x-www-form-urlencoded;char set=utf-8`

or through Jsonp GETs as follows:

- URL path: the value of `mc.GaeChannel.BASE_URL_PATH` on the client and `channel._config.BASE_URL_PATH` on the server.

- Query parameters:

    - `zx` (value of `goog.Uri.RANDOM_PARAM`): a random value to make URI unique.

    - `cid`: client ID.

    - `m`: messages, a JSON-encoded object where keys are service names and values are arrays of messages.

    - `callback`: name of the Jsonp callback function.


## Messages sent from server to client ##

Messages are sent JSON-encoded either though GAE Channel or as response to a forward channel request sent by the client. The structure of JSON data is as follows:

    {
        '<Service name 1>': <Array of messages>,
        '<Service name 2>': <Array of messages>,
        ...
    }

Response to a forward channel request can be empty, a message sent over GAE Channel is expected to be non-empty.


## System service messages ##

When messages need to be sent over the channel for the channel's internal purposes, this is done though a dedicated system service. The name of the service is `'system'` (the value of `mc.GaeChannel.SYSTEM_SERVICE_` constant of the client and `channel._SYSTEM_SERVICE` constant on the server).


### Opening backward channel ###

To be able to receive messages from the server through GAE Channel or by polling, the client needs to open the backward channel. The flow begins with client sending to the server a message `{'getToken': <boolean>}` over the system service. The boolean value is used to force polling. When the server receives the message, if the value is `false`, it will try to create a GAE channel, and if this fails because of quota depletion or any other error, will fall back to polling. If the value is `true`, it will use polling without trying to open a GAE channel.

The server then responds over the system service with message `{'opened': {'token': '<token>', 'poll': <boolean>}}`. `'poll'` value is used to indicate whether polling or GAE Channel will be used as a mechanism to send updates to the client. Depending on the mechanism, `<token>` will either be the one provided by GAE Channel API, or a random string generated internally that the client should use to authenticate itself when polling the server, and which like the GAE token expires in two hours.


### Pickups ###

The server can send a message `{'pickup': <pickup ID>}` to the client over the system service, which instructs the client to send the same message `{'pickup': <pickup ID>}` over the system service to the server. This allows us to send data to the client in the response to a forward channel request in cases when the data is too large to be sent over GAE Channel.


### Polling ###

To poll the server, the client should send a message `{'poll': <token previously sent by the server>}`. The server will respond with any messages sent to the client since it was last seen.
