# Storage #

As mentioned in [Data Synchronization](https://github.com/Basepoon/mightyclient/blob/master/wiki/synchronization.md), the server-side setup requires you to have a class that implements `abstract_storage.AbstractStorage`. This is a class to which datasource service delegates all tasks that have to do with persistently storing and retrieving data.

The two things that a storage class should know how to do is to retrieve the state of the data (`get_disk_state` method) and to apply a set of updates (`save_updates` method).

`get_disk_state` is called when a client connects to the datasource and we need to send to it the state of the data to initialize an `mc.RootNode` instance with. We will not call this method when the client temporarily disconnects and then re-connects: that that case we load from memcache a sequence of updates between the client's version and the current version, and send them to the client.

`save_updates` method is called when there are updates that need to be saved. Aside from the updates themselves (each update is a combination of a path and a new value at this path), we pass to this method the version of the data that would result from the updates. To avoid corruption of the data, this method should apply the updates atomically and only over the correct previous state of the data, i.e. the one whose version is supplied version minus # of updates (version is incremented by 1 by each update). Even if there are many messages with updates sent by clients per second, calls to `save_updates` would still be made one at a time. The updates are batched together in memcache, and while a batch is being saved, a lock (part of the in-memory state memcache item) is set that prevents a new call to `save_updates` from being made until the previous one finishes.

The [demo app](http://mightyclient.appspot.com/) contains a sample implementation `datastore_storage.DatastoreStorage` which stores each value of the data tree in a separate 'entity' in GAE Datastore. Other possible implementations include storing the data in application-specific datastore Models or storing it on a completely different server using urlfetch.

Please refer to the documentation of AbstractStorage class for more details.
