# Data Synchronization #

Data synchronization is implemented in modules `client/mc/serveragent.js` on the client and `server/mc/datasource.py` on the server.


## Client-side API ##

On the client the synchronization logic is encapsulated in the class `mc.ServerAgent`. An instance of this class syncs an instance of [`mc.ds.RootNode`](https://github.com/Basepoon/mightyclient/blob/master/wiki/datasource.md) with the server via an instance of [`mc.GaeChannel`](https://github.com/Basepoon/mightyclient/blob/master/wiki/channel.md). The initialization process is first to create the channel, then create a ServerAgent, passing to the constructor the channel and a service name. As soon as the channel is open, the ServerAgent requests the initial state of the data from the server, and once the initial state is loaded, instantiates `mc.ds.RootNode` and dispatches `INITIALIZED` event. At this point the `RootNode` becomes accessible via method `getRootNode`.

When `RootNode`'s data is modified, the following sequence of events is triggered:

- The ServerAgent dispatches `SAVED_STATUS_CHANGE` event and the method `areChangesSaved` starts returning False.

- In the next cycle of browser event processing, the updates are sent to the server.

- We then wait to receive message from the server saying which updates have been committed in-memory and which ones weren't. If there is a conflict with changes made by another client, the server agent will roll them back by setting the values to what they were before using token object `mc.ServerAgent.ROLLBACK` as event source, and will dispatch `mc.ServerAgent.EventType.ROLLBACK` event.

- Assuming the changes were successfully committed in-memory, a little later the server will let us know that they have been successfully saved to disk, the server agent will dispatch `mc.ServerAgent.EventType.SAVED_STATUS_CHANGE` event the second time, and `areChangesSaved` will start returning `True`.

Also, at any time updates made on another client can be received from the server. They are applied with token object `mc.ServerAgent.SERVER` as event source.

If the channel closes, the ServerAgent will roll back any unsaved changes and when the channel reopens, it will check with the server for any missed updates. While the channel is closed, the data is read only (ServerAgent will throw an assertion error if you try to set a value).

If expected response to some message that ServerAgent sends to the server is not received within certain timeout despite the channel being open all this time, or if the server sends an error notification, ServerAgent will dispatch an `ERROR` event. You would then either reload the page or re-initialize ServerAgent, the data tree nodes and any dependent UI elements.


## Server-side API ##

On the server you need to set up a service to which `mc.ServerAgent`s will connect. This is done by defining in `appengine_config.py` a subclass of `mc.datasource.Datasource` (which itself is a subclass of `mc.service.Service`). The reason why it's necessary to subclass `mc.datasource.Datasource` is that you need to override at least one method of that class - `get_storage`. Let's take a look at the `appengine_config.py` of the [demo app](http://mightyclient.appspot.com/):

    from google.appengine.ext import ndb

    from lib.server.mc.datasource import DatasourceService
    from server.datastore_storage import DatastoreStorage


    class DemoDatasource(DatasourceService):
      def get_storage(self):
        return DatastoreStorage(ndb.Key('Datasource', self.name[5:]))


    mc_channel_SERVICES = [
      ('demo/.*', DemoDatasource),
    ]

`get_storage` method returns an object that is capable of permanently persisting data, an instance of a class that implements `abstract_storage.AbstractStorage`. For more details on this abstract class, see [Storage](https://github.com/Basepoon/mightyclient/blob/master/wiki/storage.md).

Optionally, you can also override the method `get_access_mode` which returns the level of access granted to the client (read-only, read-write or none).
